clear all
close all
clc
%%%%%%%%%%
% Intelligence Artificielle & Machine Learning pour la modlisation de sries temporelles et de signaux 
% Master 1 Parcours IA ENS Paris Saclay
% 
% Cours 3 : Modles de signaux
%
% L. Oudre


load('signalCours3')
N=length(x);

disp(['Sampling frequency : ' num2str(Fe)])

figure
plot(t,x)
datetick('x','mm/yy')
xlim([t(1) t(end)])
xlabel('Time')
ylabel('Thousands')
title('Enplanements for U.S. Air Carrier Domestic')

%% Renormalisation
x=x-mean(x);
x=x/std(x);



%% Modele source bruit

% Ajout d'un bruit blanc additif gaussien
sigma2=0.4;
b=sqrt(sigma2)*randn(size(x));
y=x+b;

%Trace des signaux
figure
plot(t,x,t,y)
datetick('x','mm/yy')
xlim([t(1) t(end)])
xlabel('Time')

% Estimation de la DSP par periodogramme
[X,f]=TFD(x,Fe);
[Y,f]=TFD(y,Fe);
figure
plot(f,(abs(X).^2)/N,f,(abs(Y).^2)/N)
xlabel('Frequency')

%% Modele harmonique

% Fonction d'autocorrelation
lmax=50;
[gamma,l]=xcorr(x,x,lmax,'biased');
figure
stem(l,gamma)
ylabel('\gamma_x[m]')
xlabel('Lag m')

% Estimation de la DSP par periodogramme
[X,f]=TFD(x,Fe);
figure
plot(f,(abs(X).^2)/N^2)
xlabel('Frequency')

% Estimation des paramtres
f0=1/12; % frequence fondamentale repre sur la DSP
t2=(0:N-1)';
D=[ones(N,1)];
for i=1:5 % Calcul pour 5 harmoniques des a_i et phi_i
D=[D , cos(2*pi*i*f0*t2) , sin(2*pi*i*f0*t2)];
end
beta=inv(D'*D)*D'*x; 
y=D*beta; % reconstruction
figure
plot(t,x,t,y)
legend({'Signal original','Reconstruction par modele harmonique'})
datetick('x','mm/yy')
xlim([t(1) t(end)])
xlabel('Time')

%% Modele tendance et saisonnalite

% Estimation des paramtres
f0=1/12; % frequence fondamentale repre sur la DSP
t2=(0:N-1)';
D=[ones(N,1) t2 t2.^2 t2.^3 t2.^4 t2.^5]; % Ajout des tendances
for i=1:5 % Calcul pour 5 harmoniques des a_i et phi_i
D=[D , cos(2*pi*i*f0*t2) , sin(2*pi*i*f0*t2)];
end
beta=inv(D'*D)*D'*x;
y=D*beta; % reconstruction
figure
plot(t,x,t,y)
legend({'Signal original','Reconstruction par modele tendance et saisonnalite'})
datetick('x','mm/yy')
xlim([t(1) t(end)])
xlabel('Time')

% Visualisation des deux composantes
y1=D(:,1:6)*beta(1:6);
y2=D(:,7:16)*beta(7:16);
figure
plot(t,y1,t,y2)
legend({'Tendance','Saisonnalite'})
datetick('x','mm/yy')
xlim([t(1) t(end)])
xlabel('Time')


% Visualisation de l'erreur de modlisation
figure
plot(t,x-y)
hold on
line([t(1) t(end)],[0 0],'Color','r')
set(gca,'XTick',t(1:6:end))
datetick('x','mm/yy','keepticks')
xlim([t(1) t(end)])
xlabel('Time')
title('Erreur de modlisation')


%% Modele AR

% Estimation des parametres
p=24;
[a,sigma_e]=estimation_AR(x,p);

% Visualisation de l'erreur de prediction
y=filter([1 ; a],1,x);
figure
plot(t,y)
datetick('x','mm/yy')
xlim([t(1) t(end)])
xlabel('Time')
title('Erreur de prdiction AR(24)')

% Prediction des 20 derniers chantillons grace au modele
h=20;
y=x(1:end-h);
pred=prediction_AR(y,a,h);
y=[y ; pred];
figure
plot(t,x,t,y)
legend({'Signal original','Prdiction par modle AR'})
datetick('x','mm/yy')
xlim([t(end-2*h) t(end)])
xlabel('Time')
