clear all
close all
clc
%%%%%%%%%%
% Intelligence Artificielle & Machine Learning pour la modlisation de sries temporelles et de signaux 
% Master 1 Parcours IA ENS Paris Saclay
% 
% Cours 4 : Pr-traitements pour les signaux
%
% L. Oudre


load('signalCours4')
N=length(x);

disp(['Sampling frequency : ' num2str(Fe)])

figure
plot(t,x)
xlabel('Time (s)')
title('Signal original')


%% Debruitage

% Trace du signal bruite
figure
plot(t,y1)
xlabel('Time (s)')
title('Signal bruit')

% Estimation de la DSP
[Y1,f]=TFD(y1,Fe);
figure
plot(f,(abs(Y1).^2)/N)
xlabel('Frequency')
title('Estimation de la DSP du signal bruit')

% Filtrage passe-bas
fc=10;
z1=filtrage(y1,'low',fc,Fe);
figure
plot(t,x,t,z1)
xlabel('Time (s)')
legend({'Signal original','Debruitage par filtre passe-bas'})

% Filtrage de Wiener
sigma2=0.1;
Y1=fft(y1);
Z1=Y1.*((abs(Y1).^2)./(abs(Y1).^2 + N*sigma2));
z1=ifft(Z1);
figure
plot(t,x,t,z1)
xlabel('Time (s)')
legend({'Signal original','Debruitage par filtre de Wiener'})

%% Detrending

figure
plot(t,y2)
xlabel('Time (s)')
title('Signal avec tendance')


% Regression
D=[ones(N,1) t t.^2 t.^3 t.^4 t.^5]; %Tendances polynomiales
beta=inv(D'*D)*D'*y2;
tendance=D*beta;
z2=y2-tendance;

figure
plot(t,x,t,z2)
xlabel('Time (s)')
legend({'Signal original','Detrending par rgression'})

%% Bruit impulsionnel

figure
plot(t,y3)
xlabel('Time (s)')
title('Signal avec bruit impulsionnel')

% Filtrage median
z3=medfilt1(y3,3);
figure
plot(t,x,t,z3)
xlabel('Time (s)')
legend({'Signal original','Dbruitage par filtrage mdian'})

% Detection par modele AR
p=10; % ordre du modele
b=1; %parametre de fusion
K=1; % seuil de detection
[a,sigma_e]=estimation_AR(y3,p);
T=detection_phase(y3,K,b,p,a,sigma_e);
figure
plot(t,y3)
hold on
plot(t(T),y3(T),'og')
xlabel('Time (s)')
legend({'Signal original','Detection du bruit par modle AR'})

%% Interpolation

n=sort(n); % Set d'echantillons manquants

% Interpolation lineaire (echantillons manquants connus)
T2=zeros(N,1);
T2(n)=1;% 
recons= interp1(t(~T2),y3(~T2),t(find(T2)),'linear');
z3=y3;
z3(n)=recons;
figure
plot(t,x,t,z3)
xlabel('Time (s)')
title('Interpolation lineaire (echantillons manquants connus)')
legend({'Signal original','Signal interpol'})

% Interpolation par modele AR (echantillons manquants connus)
p=10; % ordre du modele
[a,sigma_e]=estimation_AR(y3,p);
[recons]=interpolation_phase(y3,n,p,a);
z3=y3;
z3(n)=recons;
figure
plot(t,x,t,z3)
xlabel('Time (s)')
title('Interpolation par modle AR (echantillons manquants connus)')
legend({'Signal original','Signal interpol'})

% Interpolation par modle AR (echantillons manquants non connus cf
% paragraphe precedent)
p=10; % ordre du modele
[a,sigma_e]=estimation_AR(y3,p);
[recons]=interpolation_phase(y3,T,p,a);
z3=y3;
z3(T)=recons;
figure
plot(t,x,t,z3)
xlabel('Time (s)')
title('Interpolation par modle AR (echantillons manquants non connus)')
legend({'Signal original','Signal interpol'})

%% Detection de ruptures

figure
plot(t,y4)
xlabel('Time (s)')
title('Signal avec non-stationnarits')


K=3;
[L]=detecte_rupture(y4,K);

figure
% vraies ruptures en noir
% ruptures detectees en rouge
plot(t,y4)
hold on
for i=1:length(n2)
    line([t(n2(i)) t(n2(i))],[-1 2.5],'Color','k')
end
for i=1:length(L)
    line([t(L(i)) t(L(i))],[-1 2.5],'Color','r')
end
xlabel('Time (s)')
legend({'Signal original','Vraies ruptures (noir)','Ruptures dtectes (rouge)'})

