clear all
close all
clc
%%%%%%%%%%
% Intelligence Artificielle & Machine Learning pour la modlisation de sries temporelles et de signaux 
% Master 1 Parcours IA ENS Paris Saclay
% 
% Cours 5 : Reprsentation des signaux (partie 1)
%
% L. Oudre


load('signalCours5')
N=length(x);

disp(['Sampling frequency : ' num2str(Fe)])

figure
plot(t,x)
xlim([t(1) t(end)])
ylabel('Degrees Celsius')
xlabel('Year')
title('Annual mean temperature anomalies relative to a base period')


% TFD du signal
[X,f]=TFD(x,Fe);
figure
stem(f,abs(X).^2)
xlabel('Frequency (Hz)')
xlim([0 Fe/2])

% Suppression des coefficients proches de zero et reconstruction
X=fft(x);
X(abs(X).^2<1)=0;
pourcentage=sum(X~=0)/length(X)
y=ifft(X);
figure
plot(t,x,t,y)
legend({'Signal original','Reconstruction par TFD inverse'})
sum(abs(y-x))


% Dictionnaire de Fourier
D=[ones(1,N)];
for i=1:N/2
    f=i*Fe/N;
D=[D ; cos(2*pi*f*t) ; sin(2*pi*f*t)];
end
D=D';

% Sparse coding (GD) avec k composantes
k=50;
[Z]=sparse_coding(x,D,k);
y=D*Z;
figure
plot(t,x,t,y)
legend({'Signal original','Reconstruction par codage parcimonieux (GD)'})
sum(abs(y-x))

% Trace de la RMSE en fonction du nombre de composantes gardees (GD)
for i=1:size(D,1)
    [Z]=sparse_coding(x,D,i);
    y=D*Z;
    err(i)=mean(abs(y-x).^2);
    spar(i)=sum(Z~=0)/length(Z);
end
figure
plot(spar,err)
ylim([0 0.1])
xlabel('% de composantes gardes')
ylabel('RMSE')
title('Codage parcimonieux par descente de gradient')


% Sparse coding (Lasso) avec k composantes
lambda=0.005;
[Z,Fit] = lasso(D,x,'Lambda',lambda);
y=D*Z+Fit.Intercept;
sum(Z~=0)
figure
plot(t,x,t,y)
legend({'Signal original','Reconstruction par codage parcimonieux (Lasso)'})
sum(abs(y-x))


% Trace de la RMSE en fonction du nombre de composantes gardees (Lasso)
[Z,Fit] = lasso(D,x,'RelTol',1e-7);
for i=1:size(Z,2);
    y=D*Z(:,i)+Fit.Intercept(i);
    err2(i)=mean(abs(y-x).^2);
    spar2(i)=sum(Z(:,i)~=0)/length(Z(:,i));
end
figure
plot(spar2,err2)
ylim([0 0.1])
xlabel('% de composantes gardes')
ylabel('RMSE')
title('Codage parcimonieux par Lasso')


% Mise du signal en trames
Nwindow=32;
Noverlap=7*Nwindow/8;
X=create_frames(x,Nwindow,Noverlap);

% Initialisation du dictionnaire
k=5; % nombre d'atomes
D=randn(Nwindow,k); % initialisation aleatoire
D = D - repmat( mean(D), [size(D,1) , 1] ); % moyenne nulle
D = D ./ repmat( sqrt(sum(D.^2)), [size(D,1), 1] ); % variance unitaire

% Apprentissage de dictionnaire
for i=1:100
[Z]=sparse_coding(X,D,k);
[D]=dictionnary_learning(X,Z,D);
end

% Trace des atomes appris
d_max=max(max(D));
d_min=min(min(D));
figure
for i=1:k
    subplot(k,1,i)
    plot(D(:,i))
    ylim([d_min d_max])
end

% Trace des activations
z_max=max(max(Z));
z_min=min(min(Z));
figure
for i=1:k
    subplot(k,1,i)
    bar(Z(i,:))
    ylim([z_min z_max])
end

% Reconstruction
X_hat=D*Z;
x_hat=icreate_frames(X_hat,Noverlap);
figure
plot((1:length(x_hat)),x(1:length(x_hat)),(1:length(x_hat)),x_hat)
legend({'Signal original','Reconstruction par dictionnaire'})




