clear all
close all
clc
%%%%%%%%%%
% Intelligence Artificielle & Machine Learning pour la modlisation de sries temporelles et de signaux 
% Master 1 Parcours IA ENS Paris Saclay
% 
% Cours 5 : Reprsentation des signaux (partie 2)
%
% L. Oudre


load('signalCours5b')
addpath(genpath([pwd '\sporco-m0.0.9\']))
N=length(x);

disp(['Sampling frequency : ' num2str(Fe)])

figure
plot(t,x)
xlim([t(1) t(end)])


% Mise du signal en trames
Nwindow=32;
Noverlap=7*Nwindow/8;
X=create_frames(x,Nwindow,Noverlap);


%% Sparse coding

% Initialisation du dictionnaire
k=5; % nombre d'atomes
D=randn(Nwindow,k); % initialisation aleatoire
D = D - repmat( mean(D), [size(D,1) , 1] ); % moyenne nulle
D = D ./ repmat( sqrt(sum(D.^2)), [size(D,1), 1] ); % variance unitaire

% Apprentissage de dictionnaire
for i=1:100
[Z]=sparse_coding(X,D,k);
[D]=dictionnary_learning(X,Z,D);
end

% Trace des atomes appris
d_max=max(max(D));
d_min=min(min(D));
figure
for i=1:k
    subplot(k,1,i)
    plot(D(:,i))
    ylim([d_min d_max])
end

% Trace des activations
z_max=max(max(Z));
z_min=min(min(Z));
figure
for i=1:k
    subplot(k,1,i)
    bar(Z(i,:))
    ylim([z_min z_max])
end

% Reconstruction
X_hat=D*Z;
x_hat=icreate_frames(X_hat,Noverlap);
figure
plot((1:length(x_hat)),x(1:length(x_hat)),(1:length(x_hat)),x_hat)



%% Convolutional sparse coding

D=randn(Nwindow,k); % initialisation aleatoire
D = D - repmat( mean(D), [size(D,1) , 1] ); % moyenne nulle
D = D ./ repmat( sqrt(sum(D.^2)), [size(D,1), 1] ); % variance unitaire
[D,Z]=convolutional_sparse_coding(x,D);

% Trace des atomes appris
d_max=max(max(D));
d_min=min(min(D));
figure
for i=1:k
    subplot(k,1,i)
    plot(D(:,i))
    ylim([d_min d_max])
end

% Trace des activations
z_max=max(max(Z));
z_min=min(min(Z));
figure
for i=1:k
    subplot(k,1,i)
    bar(Z(i,:))
    ylim([z_min z_max])
end

% Trace des reconstructions pour chaque atome
X_hat=zeros(k,length(x));
for i=1:k
    y_hat=conv(Z(i,:),D(:,i),'full');
    X_hat(i,:) = y_hat(1:length(x));
end
x_max=max(max(X_hat));
x_min=min(min(X_hat));
figure
for i=1:k
    subplot(k,1,i)
    plot(X_hat(i,:))
    ylim([x_min x_max])
end

% Reconstruction
x_hat=sum(X_hat);
figure
plot((1:length(x_hat)),x(1:length(x_hat)),(1:length(x_hat)),x_hat)

