clear all
close all
clc
%%%%%%%%%%
% Intelligence Artificielle & Machine Learning pour la modlisation de sries temporelles et de signaux 
% Master 1 Parcours IA ENS Paris Saclay
% 
% Cours 6 : Extraction et slection de caractristiques
%
% L. Oudre


load('signalCours6')
N=size(X1,1); % Taille des signaux
M=size(X1,2); % Nombre de signaux par classe


% X1 : donnees de la classe 1
% X2 : donnees de la classe 2


% Trace de quelques exemples
figure
for i=1:10
    subplot(5,2,i)
    if mod(i,2)==1
        plot(t,X1(:,i))
    else
        plot(t,X2(:,i))
    end
end

X=[X1 X2];


% Calcul de quelques features basiques : moyenne, variance, RMSE

Y=[mean(X) ; var(X) ; sqrt(sum(abs(X).^2))];

% Trace des moyennes, variances et RMSE
figure
subplot(1,3,1)
boxplot([Y(1,1:M) ; Y(1,M+1:end)]')
xlabel('Classes')
ylabel('Repartition moyenne')
subplot(1,3,2)
boxplot([Y(2,1:M) ; Y(2,M+1:end)]')
xlabel('Classes')
ylabel('Repartition variance')
subplot(1,3,3)
boxplot([Y(3,1:M) ; Y(3,M+1:end)]')
xlabel('Classes')
ylabel('Repartition RMSE')

nb_band=5;
Y2=zeros(nb_band,2*M);
for i=1:2*M
    Y2(:,i)=relative_energy(X(:,i),nb_band);
end

% Trace des energies relatives
figure
subplot(1,2,1)
boxplot(Y2(:,1:M)')
xlabel('Bandes de frequences')
title('Classe 1')
ylabel('Energie relative')
subplot(1,2,2)
boxplot(Y2(:,M+1:end)')
xlabel('Bandes de frequences')
title('Classe 2')
ylabel('Energie relative')


%% Approches non supervisees

% Concatenation de toutes les features
Y3=[Y ; Y2];
Y4=zscore(Y3')'; % normalisation


% Visualisation du cercle des correlations
Variables={'moyenne','variance','RMSE','E_1','E_2','E_3','E_4','E_5'};
[coeff,score,latent]=pca(Y4');
figure
biplot(coeff(:,1:2),'scores',score(:,1:2),'varlabels',Variables);
hold on
[x,y,z] = cylinder(1,200);
plot(x(1,:),y(1,:))
axis equal

% Utilisation des K-Means sur donnees brutes
[idx,C,sumd]=kmeans(Y4',2,'Replicates',10);
visualisation_kmeans(Y4,idx);

% Utilisation uniquement des features utiles
Y5=Y4([4 7],:);
[idx,C,sumd]=kmeans(Y5',2,'Replicates',10);
visualisation_kmeans(Y5,idx);

%% Approches supervisees

y=[-ones(M,1) ; ones(M,1)];

% Approche filter : coefficient de corrlation avec chacune des features
covcoeff=zeros(size(Y4,1),1);
for i=1:size(Y4,1)
    covcoeff(i)=corr(Y4(i,:)',y);
end

% Approche wrapper : test de toutes les features avec 1-NN
scoreclassif=zeros(size(Y4,1),1);
for i=1:size(Y4,1)
mdl = fitcknn(Y4(i,:)',y,'Distance','euclidean','NumNeighbors',1); 
cvmodel = crossval(mdl,'Leaveout','on');
scoreclassif(i) = 1-kfoldLoss(cvmodel);
end

% Approche embedded : algorithme LASSO
lambda=0.99;
z=lasso(Y4',y,'Lambda',lambda);
z
100*sum(sign(Y4'*z)==y)/(2*M)




