function T=detection_phase(x,K,b,p,a,sigma_e)
%%% Perform the detection of degraded samples
%%% Input parameters
%     - x : sound file
%     - K : detection threshold
%     - b : fusion parameter
%     - p : order of the AR model
%     - a : estimated AR parameters
%     - sigma_e : estimated standard deviation of the excitation signal
%%% Output parameter
%     - T : set of degraded samples


Nw=length(x);

% Calculation of the detection criterion
d=zeros(Nw,1);
for j=p+1:Nw
    d(j)=x(j)+sum(x(j-p:j-1).*flipud(a));
end
d=abs(d);

% Detection threshold
lambda_K=K*sigma_e;
T=find(d>lambda_K);

% Fusion parameter
if length(T)>0
    T_diff=diff(T);
    [extra_samples]=find(T_diff>1 & T_diff<=b);
    for k=1:length(extra_samples)
        T=[T; T(extra_samples(k))+(1:T_diff(extra_samples(k))-1)'];
    end
    T=unique(sort(T));
end
T=T(find((T>p) & (T<Nw-p)));