function [a,sigma_e]=estimation_AR(x,p)
%% Estimation of the AR parameters based on Yule-Walker equations
%%% Input parameters
%     - x : sound file
%     - p : order of the AR model
%%% Output parameters
%     - a : estimated AR parameters
%     - sigma_e : estimated standard deviation of the excitation signal



N=length(x);

% Calculation of the autocorrelation function
R=zeros(p+1,1);
for i=1:p+1
    R(i)=sum(x(i:N).*x(1:N-(i-1)))/N;
end

% Levinson-Durbin algorithm
a=zeros(p);
sigma2=zeros(p,1);
a(1,1)=-R(2)/R(1);
sigma2(1)=(1-abs(a(1,1))^2)*R(1);

for k=2:p
    a(k,k)=-(R(k+1)+a(k-1,1:k-1)*flipud(R(2:k)))/sigma2(k-1);
    sigma2(k)=(1-abs(a(k,k))^2)*sigma2(k-1);
    for i=1:k-1
        a(k,i)=a(k-1,i)+a(k,k)*a(k-1,k-i);
    end
end

sigma_e=sqrt(sigma2(p));
a=a(p,:)';
a=a(1:p);