function [Z]=sparse_coding(X,D,k)
%% Sparse coding based on gradient descent
%%% Input parameters
%     - X : input signals
%     - D : dictionnary
%     - k : number of wanted components
%%% Output parameters
%     - Z : activations
%%% This function is inspired by the Numerical Tours by Gabriel Peyre


select = @(A,k)repmat(A(k,:), [size(A,1) 1]);
ProjX = @(X,k)X .* (abs(X) >= select(sort(abs(X), 'descend'),k));

[N1,N2]=size(X);
[~,Nd]=size(D);

niter = 100;
gamma = 1/norm(D)^2;
Z=zeros(Nd,N2);
for i=1:niter
    R = D*Z-X;
    Z = ProjX(Z - gamma * D'*R, k);
end